package com.digiwin.athena.app.service.executor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.ItemExecutorEntity;
import com.digiwin.athena.app.infra.repository.ItemExecutorRepository;
import com.digiwin.athena.app.infra.service.ItemExecutorService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: xieps
 * @Date: 2023/8/30 14:33
 * @Version 1.0
 * @Description
 */
@Service
@Log4j2
public class ItemExecutorCreateEAIService extends AbsEAIService {


    @Resource
    private ItemExecutorRepository itemExecutorRepository;


    @Resource
    private ItemExecutorService itemExecutorService;


    @Override
    public String getServiceName() {
        return ExecutorUtil.DEMO_ITEM_EXECUTOR_INFO_CREATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {

        EAIRequest request = new EAIRequest(messageBody);
        List<ItemExecutorEntity> itemExecutorInfo = request.getToList("item_executor_info", ItemExecutorEntity.class);

        //对必传参数 且 品号和执行人编号不可以重复 进行校验
        validateParameter(itemExecutorInfo);

        //进行批量新增
        itemExecutorService.saveBatch(itemExecutorInfo);

        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("item_executor_info",itemExecutorInfo));
    }


    /**
     * 对入参进行参数校验
     *
     * @param itemExecutorInfo
     * @throws DWBusinessException
     */
    private void validateParameter(List<ItemExecutorEntity> itemExecutorInfo) throws DWBusinessException {
        if (Objects.isNull(itemExecutorInfo) || itemExecutorInfo.isEmpty()) {
            throw new DWBusinessException("入参数据为空~~");
        }
        for (ItemExecutorEntity item : itemExecutorInfo) {
            if (StringUtils.isEmpty(item.getItemNo())) {
                throw new DWBusinessException("品号字段值不为空或者NULL~");
            }
            if (StringUtils.isEmpty(item.getItemName())) {
                throw new DWBusinessException("品名字段值不为空或者NULL~");
            }
            if (StringUtils.isEmpty(item.getProcurementExecutorCode())) {
                throw new DWBusinessException("执行人编号字段值不为空或者NULL~");
            }
            if (StringUtils.isEmpty(item.getProcurementExecutorName())) {
                throw new DWBusinessException("执行人名称字段值不为空或者NULL~");
            }
        }
        List<ItemExecutorEntity> distinctList = itemExecutorInfo.stream().filter(distinctByKey(item -> Stream.of(item.getItemNo(), item.getProcurementExecutorCode()).toArray())).collect(Collectors.toList());
        if (itemExecutorInfo.size() > distinctList.size()) {
            throw new DWBusinessException("品号和执行人编号入参重复~");
        }

        LambdaQueryWrapper<ItemExecutorEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ItemExecutorEntity::getTenantSid, SecurityUtil.getUserProfile().getTenantSid());
        List<ItemExecutorEntity> entityList = itemExecutorRepository.selectList(queryWrapper);

        if (!entityList.isEmpty()) {
            for (ItemExecutorEntity executorEntity : entityList) {
                String itemNo = executorEntity.getItemNo();
                String executorNo = executorEntity.getProcurementExecutorCode();
                boolean match = itemExecutorInfo.stream().anyMatch(item -> Objects.equals(item.getItemNo(), itemNo)
                        && Objects.equals(item.getProcurementExecutorCode(), executorNo));
                if (match) {
                    throw new DWBusinessException("品号和执行人编号已重复~");
                }
            }
        }
    }


    /**
     * 用于对象去重
     *
     * @param keyExtractor 需要去重的属性
     * @param <T>
     * @return
     */
    private static <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        //记录已有对象或者属性
        ConcurrentSkipListMap<Object, Boolean> skipListMap = new ConcurrentSkipListMap<>();
        return t -> skipListMap.putIfAbsent(JSON.toJSONString(keyExtractor.apply(t)), Boolean.TRUE) == null;
    }
}
