package com.digiwin.athena.app.service.supplier;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.container.exceptions.DWBusinessException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.FontSupplierEntity;
import com.digiwin.athena.app.infra.entity.ItemExecutorEntity;
import com.digiwin.athena.app.infra.repository.FontSupplierRepository;
import com.digiwin.athena.app.infra.service.FontSupplierService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import com.digiwin.athena.opt.common.security.SecurityUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentSkipListMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @auther: bk
 * @date: 2023/9/1
 */
@Log4j2
@Service
public class FontSupplierCreateEAIService extends AbsEAIService {

    @Resource
    private FontSupplierService fontSupplierService;

    @Override
    public String getServiceName() {
        return FontSupplierUtil.DEMO_FONT_ITEM_SUPPLIER_INFO_CREATE;
    }

    @Override
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        EAIRequest eaiRequest = EAIRequest.build(messageBody);
        List<FontSupplierEntity> supplierInfo = eaiRequest.getObject("item_supplier_info", FontSupplierUtil.LIST_ENTITY_SUPPLIER);
        //对必传参数 且 品号和供应商编号不可以重复 进行校验
        validateParameter(supplierInfo);

        fontSupplierService.saveBatch(supplierInfo);
        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("item_supplier_info", supplierInfo));
    }

    private void validateParameter(List<FontSupplierEntity> supplierInfo) throws DWBusinessException {
        if (Objects.isNull(supplierInfo) || supplierInfo.isEmpty()) {
            throw new DWBusinessException("入参数据为空");
        }
        for (FontSupplierEntity item : supplierInfo) {
            if (StringUtils.isEmpty(item.getItemNo())) {
                throw new DWBusinessException("品号字段值不为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getItemName())) {
                throw new DWBusinessException("品名字段值不为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getSupplierNo())) {
                throw new DWBusinessException("供应商编号字段值不为空或者NULL");
            }
            if (StringUtils.isEmpty(item.getSupplierName())) {
                throw new DWBusinessException("供应商名称字段值不为空或者NULL");
            }
        }
        List<FontSupplierEntity> distinctList = supplierInfo.stream().filter(distinctByKey(item -> Stream.of(item.getItemNo(), item.getSupplierNo()).toArray())).collect(Collectors.toList());
        if (supplierInfo.size() > distinctList.size()) {
            throw new DWBusinessException("品号和供应商编号入参重复");
        }

        LambdaQueryWrapper<FontSupplierEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(FontSupplierEntity::getTenantSid, SecurityUtil.getUserProfile().getTenantSid());
        List<FontSupplierEntity> entityList =fontSupplierService.list(queryWrapper);

        if (!entityList.isEmpty()) {
            for (FontSupplierEntity executorEntity : entityList) {
                String itemNo = executorEntity.getItemNo();
                String executorNo = executorEntity.getSupplierNo();
                boolean match = supplierInfo.stream().anyMatch(item -> Objects.equals(item.getItemNo(), itemNo)
                        && Objects.equals(item.getSupplierNo(), executorNo));
                if (match) {
                    throw new DWBusinessException("品号和供应商编号入参重复");
                }
            }
        }
    }

    /**
     * 用于对象去重
     *
     * @param keyExtractor 需要去重的属性
     * @param <T>
     * @return
     */
    private static <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        //记录已有对象或者属性
        ConcurrentSkipListMap<Object, Boolean> skipListMap = new ConcurrentSkipListMap<>();
        return t -> skipListMap.putIfAbsent(JSON.toJSONString(keyExtractor.apply(t)), Boolean.TRUE) == null;
    }
}
